from ..engine import SearchEngine
from ..config import PROXY, TIMEOUT, FAKE_USER_AGENT
from ..utils import unquote_url


class Dogpile(SearchEngine):
    '''Seaches dogpile.com'''
    def __init__(self, proxy=PROXY, timeout=TIMEOUT):
        super(Dogpile, self).__init__(proxy, timeout)
        self._base_url = 'https://www.dogpile.com'
        self.set_headers({'User-Agent':FAKE_USER_AGENT})
    
    def _selectors(self, element):
        '''Returns the appropriate CSS selector.'''
        selectors = {
            'url': 'a[class$=title]', 
            'title': 'a[class$=title]', 
            'text': {'tag':'span', 'index':-1}, 
            'links': 'div[class^=web-] div[class$=__result]', 
            'next': 'a.pagination__num--next'
        }
        return selectors[element]
    
    def _first_page(self):
        '''Returns the initial page and query.'''
        url = u'{}/serp?q={}'.format(self._base_url, self._query)
        return {'url':url, 'data':None}
    
    def _next_page(self, tags):
        '''Returns the next page URL and post data (if any)'''
        selector = self._selectors('next')
        next_page = self._get_tag_item(tags.select_one(selector), 'href')
        url = (self._base_url + next_page) if next_page else None
        return {'url':url, 'data':None}

    def _get_text(self, tag, item='text'):
        '''Returns the text of search results items.'''
        selector = self._selectors('text')
        tag = tag.select(selector['tag'])[selector['index']]
        return self._get_tag_item(tag, 'text')

    



from .results import SearchResults
from .engines import search_engines_dict
from . import output as out
from . import config as cfg


class MultipleSearchEngines(object):
    '''Uses multiple search engines.'''
    def __init__(self, engines, proxy=cfg.PROXY, timeout=cfg.TIMEOUT):
        self._engines = [
            se(proxy, timeout) 
            for se in search_engines_dict.values() 
            if se.__name__.lower() in engines
        ]
        self._filter = None

        self.ignore_duplicate_urls = False
        self.ignore_duplicate_domains = False
        self.results = SearchResults()
        self.banned_engines = []

    def set_search_operator(self, operator):
        '''Filters search results based on the operator.'''
        self._filter = operator
    
    def search(self, query, pages=cfg.SEARCH_ENGINE_RESULTS_PAGES): 
        '''Searches multiples engines and collects the results.'''
        self.results = SearchResults()
        for engine in self._engines:
            engine.ignore_duplicate_urls = self.ignore_duplicate_urls
            engine.ignore_duplicate_domains = self.ignore_duplicate_domains
            if self._filter:
                engine.set_search_operator(self._filter)
            
            engine_results = engine.search(query, pages)
            if engine.ignore_duplicate_urls:
                engine_results._results = [
                    item for item in engine_results._results 
                    if item['link'] not in self.results.links()
                ]
            if self.ignore_duplicate_domains:
                engine_results._results = [
                    item for item in engine_results._results 
                    if item['host'] not in self.results.hosts()
                ]
            self.results._results += engine_results._results

            if engine.is_banned:
                self.banned_engines.append(engine.__class__.__name__)
        return self.results
    
    def output(self, output=out.PRINT, path=None):
        '''Prints search results and/or creates report files.'''
        output = (output or '').lower()
        query = self._engines[0]._query if self._engines else u''
        if not path:
            path = cfg.OUTPUT_DIR + u'_'.join(query.split())
        out.console('')

        if out.PRINT in output:
            out.print_results(self._engines)
        if out.HTML in output:
            out.write_file(out.create_html_data(self._engines), path + u'.html') 
        if out.CSV in output:
            out.write_file(out.create_csv_data(self._engines), path + u'.csv') 
        if out.JSON in output:
            out.write_file(out.create_json_data(self._engines), path + u'.json')


class AllSearchEngines(MultipleSearchEngines):
    '''Uses all search engines.'''
    def __init__(self, proxy=cfg.PROXY, timeout=cfg.TIMEOUT):
        super(AllSearchEngines, self).__init__(
            list(search_engines_dict), proxy, timeout
        )




from ..engine import SearchEngine
from ..config import PROXY, TIMEOUT, FAKE_USER_AGENT
from ..utils import unquote_url, quote_url


class Google(SearchEngine):
    '''Searches google.com'''
    def __init__(self, proxy=PROXY, timeout=TIMEOUT):
        super(Google, self).__init__(proxy, timeout)
        self._base_url = 'https://www.google.com'
        self._delay = (2, 6)
        self._current_page = 1
        
        self.set_headers({'User-Agent':FAKE_USER_AGENT})

    def _selectors(self, element):
        '''Returns the appropriate CSS selector.'''
        selectors = {
            'url': 'a[href]', 
            'title': 'a', 
            'text': 'div[data-sncf="1"]',
            'links': 'div#search div.g', 
            'next': 'a[href][aria-label="Page {page}"]'
        }
        return selectors[element]
    
    def _first_page(self):
        '''Returns the initial page and query.'''
        url = u'{}/search?q={}&hl={}&lr=LANG_{}'.format(self._base_url, quote_url(self._query, '', "ES", "ES"))
        return {'url':url, 'data':None}
    
    def _next_page(self, tags):
        '''Returns the next page URL and post data (if any)'''
        self._current_page += 1
        selector = self._selectors('next').format(page=self._current_page)
        next_page = self._get_tag_item(tags.select_one(selector), 'href')
        url = None
        if next_page:
            url = self._base_url + next_page
        return {'url':url, 'data':None}

    def _get_url(self, tag, item='href'):
        '''Returns the URL of search results item.'''
        selector = self._selectors('url')
        url = self._get_tag_item(tag.select_one(selector), item)

        if url.startswith(u'/url?q='):
            url = url.replace(u'/url?q=', u'').split(u'&sa=')[0]
        return unquote_url(url)



from .yahoo import Yahoo
from ..config import PROXY, TIMEOUT


class Aol(Yahoo):
    '''Seaches aol.com'''
    def __init__(self, proxy=PROXY, timeout=TIMEOUT):
        super(Aol, self).__init__(proxy, timeout)
        self._base_url = u'https://search.aol.com'

    def _first_page(self):
        '''Returns the initial page and query.'''
        url_str = u'{}/aol/search?q={}&ei=UTF-8&nojs=1'
        url = url_str.format(self._base_url, self._query)
        self._http_client.get(self._base_url)
        return {'url':url, 'data':None}




import requests
from .config import PYTHON_VERSION


def quote_url(url, safe=';/?:@&=+$,#'):
    '''encodes URLs.'''
    if PYTHON_VERSION == 2:
        url = encode_str(url)
    return requests.utils.quote(url, safe=safe)

def unquote_url(url):
    '''decodes URLs.'''
    if PYTHON_VERSION == 2:
        url = encode_str(url)
    return decode_bytes(requests.utils.unquote(url))

def is_url(link):
    '''Checks if link is URL'''
    parts = requests.utils.urlparse(link)
    return bool(parts.scheme and parts.netloc)

def domain(url):
    '''Returns domain form URL'''
    host = requests.utils.urlparse(url).netloc
    return host.lower().split(':')[0].replace('www.', '')

def encode_str(s, encoding='utf-8', errors='replace'):
    '''Encodes unicode to str, str to bytes.'''
    return s if type(s) is bytes else s.encode(encoding, errors=errors)

def decode_bytes(s, encoding='utf-8', errors='replace'):
    '''Decodes bytes to str, str to unicode.'''
    return s.decode(encoding, errors=errors) if type(s) is bytes else s




from .engines import *


__title__ = 'search_engines'
__version__ = '0.5'
__author__ = 'Tasos M. Adamopoulos'

__all__ = [
    'Google', 
    'Bing', 
    'Yahoo', 
    'Aol', 
    'Duckduckgo', 
    'Startpage', 
    'Dogpile', 
    'Ask', 
    'Mojeek', 
    'Qwant',
    'Torch'
]



class SearchResults(object):
    '''Stores the search results'''
    def __init__(self, items=None):
        self._results = items or []
    
    def links(self):
        '''Returns the links found in search results'''
        return [row.get('link') for row in self._results]
    
    def titles(self):
        '''Returns the titles found in search results'''
        return [row.get('title') for row in self._results]
    
    def text(self):
        '''Returns the text found in search results'''
        return [row.get('text') for row in self._results]
    
    def hosts(self):
        '''Returns the domains found in search results'''
        return [row.get('host') for row in self._results]
    
    def results(self):
        '''Returns all data found in search results'''
        return self._results
    
    def __getitem__(self, index):
        return self._results[index]
    
    def __len__(self):
        return len(self._results)

    def __str__(self):
        return '<SearchResults ({} items)>'.format(len(self._results))
    
    def append(self, item):
        '''appends an item to the results list.'''
        self._results.append(item)
    
    def extend(self, items):
        '''appends items to the results list.'''
        self._results.extend(items)



from ..engine import SearchEngine
from ..config import PROXY, TIMEOUT


class Ask(SearchEngine):
    '''Searches ask.com'''
    def __init__(self, proxy=PROXY, timeout=TIMEOUT):
        super(Ask, self).__init__(proxy, timeout)
        self._base_url = 'https://uk.ask.com'
    
    def _selectors(self, element):
        '''Returns the appropriate CSS selector.'''
        selectors = {
            'url': 'a.PartialSearchResults-item-title-link.result-link', 
            'title': 'a.PartialSearchResults-item-title-link.result-link', 
            'text': 'p.PartialSearchResults-item-abstract', 
            'links': 'div.PartialSearchResults-body div.PartialSearchResults-item', 
            'next': 'li.PartialWebPagination-next a[href]'
        }
        return selectors[element]
    
    def _first_page(self):
        '''Returns the initial page and query.'''
        url_str = u'{}/web?o=0&l=dir&qo=serpSearchTopBox&q={}'
        url = url_str.format(self._base_url, self._query)
        return {'url':url, 'data':None}
    
    def _next_page(self, tags):
        '''Returns the next page URL and post data (if any)'''
        next_page = tags.select_one(self._selectors('next'))
        url = None
        if next_page:
            url = self._base_url + next_page['href']
        return {'url':url, 'data':None}




from ..engine import SearchEngine
from ..config import PROXY, TIMEOUT
from ..utils import unquote_url


class Yahoo(SearchEngine):
    '''Searches yahoo.com'''
    def __init__(self, proxy=PROXY, timeout=TIMEOUT):
        super(Yahoo, self).__init__(proxy, timeout)
        self._base_url = 'https://search.yahoo.com'
    
    def _selectors(self, element):
        '''Returns the appropriate CSS selector.'''
        selectors = {
            'url': 'div.compTitle h3.title a', 
            'title': 'div.compTitle h3.title', 
            'text': 'div.compText', 
            'links': 'div#web li div.dd.algo.algo-sr', 
            'next': 'a.next'
        }
        return selectors[element]
    
    def _first_page(self):
        '''Returns the initial page and query.'''
        url_str = u'{}/search?p={}&ei=UTF-8&nojs=1'
        url = url_str.format(self._base_url, self._query)
        return {'url':url, 'data':None}
    
    def _next_page(self, tags):
        '''Returns the next page URL and post data (if any)'''
        selector = self._selectors('next')
        url = self._get_tag_item(tags.select_one(selector), 'href') or None
        return {'url':url, 'data':None}

    def _get_url(self, link, item='href'):
        selector = self._selectors('url')
        url = self._get_tag_item(link.select_one(selector), 'href')
        url = url.split(u'/RU=')[-1].split(u'/R')[0]
        return unquote_url(url)

    def _get_title(self, tag, item='text'):
        '''Returns the title of search results items.'''
        title = tag.select_one(self._selectors('title'))
        for span in title.select('span'):
            span.decompose()
        return self._get_tag_item(title, item)

    



from bs4 import BeautifulSoup
from time import sleep
from random import uniform as random_uniform
from collections import namedtuple

from .results import SearchResults
from .http_client import HttpClient
from . import utils
from . import output as out
from . import config as cfg

class SearchEngine(object):
    '''The base class for all Search Engines.'''
    def __init__(self, proxy=cfg.PROXY, timeout=cfg.TIMEOUT, username: str or None = None, password: str or None = None):
        '''
        :param str proxy: optional, a proxy server  
        :param int timeout: optional, the HTTP timeout
        '''
        self._http_client = HttpClient(timeout, proxy, username, password) 
        self._delay = (1, 4)
        self._query = ''
        self._filters = []

        self.results = SearchResults()
        '''The search results.'''
        self.ignore_duplicate_urls = False
        '''Collects only unique URLs.'''
        self.ignore_duplicate_domains = False
        '''Collects only unique domains.'''
        self.is_banned = False
        '''Indicates if a ban occured'''

    def _selectors(self, element):
        '''Returns the appropriate CSS selector.'''
        raise NotImplementedError()
    
    def _first_page(self):
        '''Returns the initial page URL.'''
        raise NotImplementedError()
    
    def _next_page(self, tags):
        '''Returns the next page URL and post data.'''
        raise NotImplementedError()
    
    def _get_url(self, tag, item='href'):
        '''Returns the URL of search results items.'''
        selector = self._selectors('url')
        url = self._get_tag_item(tag.select_one(selector), item)
        return utils.unquote_url(url)
    
    def _get_title(self, tag, item='text'):
        '''Returns the title of search results items.'''
        selector = self._selectors('title')
        return self._get_tag_item(tag.select_one(selector), item)
    
    def _get_text(self, tag, item='text'):
        '''Returns the text of search results items.'''
        selector = self._selectors('text')
        return self._get_tag_item(tag.select_one(selector), item)
    
    def _get_page(self, page, data=None):
        '''Gets pagination links.'''
        if data:
            return self._http_client.post(page, data)
        return self._http_client.get(page)
    
    def _get_tag_item(self, tag, item):
        '''Returns Tag attributes.'''
        if not tag:
            return u''
        return tag.text if item == 'text' else tag.get(item, u'')

    def _item(self, link):
        '''Returns a dictionary of the link data.'''
        return {
            'host': utils.domain(self._get_url(link)), 
            'link': self._get_url(link), 
            'title': self._get_title(link).strip(), 
            'text': self._get_text(link).strip()
        } 

    def _query_in(self, item):
        '''Checks if query is contained in the item.'''
        return self._query.lower() in item.lower()
    
    def _filter_results(self, soup):
        '''Processes and filters the search results.''' 
        tags = soup.select(self._selectors('links'))
        results = [self._item(l) for l in tags]

        if u'url' in self._filters:
            results = [l for l in results if self._query_in(l['link'])]
        if u'title' in self._filters:
            results = [l for l in results if self._query_in(l['title'])]
        if u'text' in self._filters:
            results = [l for l in results if self._query_in(l['text'])]
        if u'host' in self._filters:
            results = [l for l in results if self._query_in(utils.domain(l['link']))]
        return results
    
    def _collect_results(self, items):
        '''Colects the search results items.''' 
        for item in items:
            if not utils.is_url(item['link']):
                continue
            if item in self.results:
                continue
            if self.ignore_duplicate_urls and item['link'] in self.results.links():
                continue
            if self.ignore_duplicate_domains and item['host'] in self.results.hosts():
                continue
            self.results.append(item)

    def _is_ok(self, response):
        '''Checks if the HTTP response is 200 OK.'''
        self.is_banned = response.http in [403, 429, 503]
        
        if response.http == 200:
            return True
        msg = ('HTTP ' + str(response.http)) if response.http else response.html
        out.console(msg, level=out.Level.error)
        return False
    
    def set_headers(self, headers):
        '''Sets HTTP headers.
        
        :param headers: dict The headers 
        '''
        self._http_client.session.headers.update(headers)
    
    def set_search_operator(self, operator):
        '''Filters search results based on the operator. 
        Supported operators: 'url', 'title', 'text', 'host'

        :param operator: str The search operator(s)
        '''
        operators = utils.decode_bytes(operator or u'').lower().split(u',')
        supported_operators = [u'url', u'title', u'text', u'host']

        for operator in operators:
            if operator not in supported_operators:
                msg = u'Ignoring unsupported operator "{}"'.format(operator)
                out.console(msg, level=out.Level.warning)
            else:
                self._filters += [operator]
    
    def search(self, query, pages=cfg.SEARCH_ENGINE_RESULTS_PAGES): 
        '''Queries the search engine, goes through the pages and collects the results.
        
        :param query: str The search query  
        :param pages: int Optional, the maximum number of results pages to search  
        :returns SearchResults object
        '''
        out.console('Searching {}'.format(self.__class__.__name__))
        self._query = utils.decode_bytes(query)
        self.results = SearchResults()
        request = self._first_page()

        for page in range(1, pages + 1):
            try:
                response = self._get_page(request['url'], request['data'])
                if not self._is_ok(response):
                    break
                tags = BeautifulSoup(response.html, "html.parser")
                items = self._filter_results(tags)
                self._collect_results(items)
                
                msg = 'page: {:<8} links: {}'.format(page, len(self.results))
                out.console(msg, end='')
                request = self._next_page(tags)

                if not request['url']:
                    break
                if page < pages:
                    sleep(random_uniform(*self._delay))
            except KeyboardInterrupt:
                break
        out.console('', end='')
        return self.results
    
    def output(self, output=out.PRINT, path=None):
        '''Prints search results and/or creates report files.
        Supported output format: html, csv, json.
        
        :param output: str Optional, the output format  
        :param path: str Optional, the file to save the report  
        '''
        output = (output or '').lower()
        if not path:
            path = cfg.os_path.join(cfg.OUTPUT_DIR, u'_'.join(self._query.split()))
        out.console('')

        if out.PRINT in output:
            out.print_results([self])
        if out.HTML in output:
            out.write_file(out.create_html_data([self]), path + u'.html') 
        if out.CSV in output:
            out.write_file(out.create_csv_data([self]), path + u'.csv') 
        if out.JSON in output:
            out.write_file(out.create_json_data([self]), path + u'.json')



from bs4 import BeautifulSoup

from search_engines.engine import SearchEngine
from search_engines.config import PROXY, TIMEOUT, FAKE_USER_AGENT


class Metager(SearchEngine):
    '''Searches metager.org'''

    def __init__(self, proxy=PROXY, timeout=TIMEOUT):
        super(Metager, self).__init__(proxy, timeout)
        self._base_url = 'https://metager.org'
        self.set_headers({'User-Agent': FAKE_USER_AGENT})

    def _selectors(self, element):
        """Returns the appropriate CSS selector."""
        selectors = {
            'url': 'a.result-link',
            'title': 'h2.result-title a',
            'text': 'div.result-description',
            'links': '#results div.result',
            'next': '#next-search-link a',
            }
        return selectors[element]

    def redirect(self, query):
        '''Redirects initial request to actual result page.'''
        response = self._get_page(query)
        src_page = BeautifulSoup(response.html, "html.parser")
        url = src_page.select_one('iframe').get('src')

        return url

    def _first_page(self):
        '''Returns the initial page and query.'''
        query = f'{self._base_url}/meta/meta.ger3?eingabe={self._query}'
        url = self.redirect(query)

        return {'url': url, 'data': None}

    def _next_page(self, tags):
        '''Returns the next page URL.'''
        next_page = tags.select_one(self._selectors('next'))
        url = None
        if next_page:
            url = self.redirect(next_page['href'])

        return {'url': url, 'data': None}



from json import loads

from ..engine import SearchEngine
from ..config import PROXY, TIMEOUT
from ..utils import unquote_url


class Qwant(SearchEngine):
    '''Searches qwant.com'''
    def __init__(self, proxy=PROXY, timeout=TIMEOUT):
        super(Qwant, self).__init__(proxy, timeout)
        self._base_url = u'https://api.qwant.com/v3/search/web?q={}&count=10&locale=en_US&offset={}&device=desktop&safesearch=1'
        self._offset = 0
        self._max_offset = 50
        
    def _selectors(self, element):
        '''Returns the appropriate CSS selector.'''
        selectors = {
            'url': 'url', 
            'title': 'title', 
            'text': 'desc', 
            'links': ['data', 'result', 'items', 'mainline']
        }
        return selectors[element]
    
    def _first_page(self):
        '''Returns the initial page and query.'''
        url = self._base_url.format(self._query, self._offset)
        return {'url':url, 'data':None}
    
    def _next_page(self, tags):
        '''Returns the next page URL and post data (if any)'''
        self._offset += 10
        url = None
        status = loads(tags.get_text())['status']
        if status == 'success' and self._offset <= self._max_offset:
            url = self._base_url.format(self._query, self._offset)
        return {'url':url, 'data':None}

    def _get_url(self, tag, item='href'):
        '''Returns the URL of search results item.'''
        return unquote_url(tag.get(self._selectors('url'), u''))
    
    def _get_title(self, tag, item='text'):
        '''Returns the title of search results items.'''
        return tag.get(self._selectors('title'), u'')
    
    def _get_text(self, tag, item='text'):
        '''Returns the text of search results items.'''
        return tag.get(self._selectors('text'), u'')
    
    def _filter_results(self, soup):
        '''Processes and filters the search results.''' 
        tags = loads(soup.get_text())['data']['result']['items']['mainline']
        tags = [j for i in tags for j in i['items'] if i['type'] != u'ads']
        results = [self._item(l) for l in tags]

        if u'url' in self._filters:
            results = [l for l in results if self._query_in(l['link'])]
        if u'title' in self._filters:
            results = [l for l in results if self._query_in(l['title'])]
        if u'text' in self._filters:
            results = [l for l in results if self._query_in(l['text'])]
        if u'host' in self._filters:
            results = [l for l in results if self._query_in(utils.domain(l['link']))]
        return results



from setuptools import setup, find_packages
import codecs 
import os 

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\\n" + fh.read()

requirements = []
with open('requirements.txt') as f:
    requirements = f.readlines()

setup(
    name="search_engines_kit",
    version='0.0.5',
    description="Search Engines Scraper",
    url = "https://github.com/Juanchobanano/Search-Engines-Scraper",
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(),
    install_requires=requirements,
    keywords=['search', 'browsers']
)


from __future__ import print_function

import csv
import json
import io
import re
from collections import namedtuple

try:
    from shutil import get_terminal_size
except ImportError:
    from .libs.get_terminal_size import get_terminal_size
    
from .utils import encode_str, decode_bytes
from .libs import windows_cmd_encoding
from .config import PYTHON_VERSION


def print_results(search_engines):
    '''Prints the search results.'''
    for engine in search_engines:
        console(engine.__class__.__name__ + u' results') 

        for i, v in enumerate(engine.results, 1):
            console(u'{:<4}{}'.format(i, v['link'])) 
        console(u'')

def create_csv_data(search_engines):
    '''CSV formats the search results.'''
    encoder = decode_bytes if PYTHON_VERSION == 3 else encode_str
    data = [['query', 'engine', 'domain', 'URL', 'title', 'text']]

    for engine in search_engines:
        for i in engine.results:
            row = [
                engine._query, engine.__class__.__name__, 
                i['host'], i['link'], i['title'], i['text']
            ]
            row = [encoder(i) for i in row]
            data.append(row)
    return data

def create_json_data(search_engines):
    '''JSON formats the search results.'''
    jobj = {
        u'query': search_engines[0]._query, 
        u'results': {
            se.__class__.__name__: [i for i in se.results] 
            for se in search_engines
        }
    }
    return json.dumps(jobj)

def create_html_data(search_engines):
    '''HTML formats the search results.'''
    query = decode_bytes(search_engines[0]._query) if search_engines else u''
    tables = u''

    for engine in search_engines:
        rows = u''
        for i, v in enumerate(engine.results, 1):
            data = u''
            if u'title' in engine._filters:
                data += HtmlTemplate.data.format(_replace_with_bold(query, v['title']))
            if u'text' in engine._filters:
                data += HtmlTemplate.data.format(_replace_with_bold(query, v['text']))
            link = _replace_with_bold(query, v['link']) if u'url' in engine._filters else v['link']
            rows += HtmlTemplate.row.format(number=i, href=v['link'], link=link, data=data)
        
        engine_name = engine.__class__.__name__
        tables += HtmlTemplate.table.format(engine=engine_name, rows=rows)
    return HtmlTemplate.html.format(query=query, table=tables)

def _replace_with_bold(query, data):
    '''Places the query in <b> tags.'''
    for match in re.findall(query, data, re.I):
        data = data.replace(match, u'<b>{}</b>'.format(match))
    return data


def write_file(data, path, encoding='utf-8'):
    '''Writes search results data to file.'''
    try:
        if PYTHON_VERSION == 2 and type(data) in (list, str):
            f = io.open(path, 'wb') 
        else: 
            f = io.open(path, 'w', encoding=encoding, newline='')
        
        if type(data) is list:
            writer = csv.writer(f)
            writer.writerows(data)
        else:
            f.write(data)
        f.close()
        console(u'Output file: ' + path)
    except IOError as e:
        console(e, level=Level.error)


def console(msg, end='\n', level=None):
    '''Prints data on the console.'''
    console_len = get_terminal_size().columns
    clear_line = u'\r{}\r'.format(u' ' * (console_len - 1))
    msg = clear_line + (level or u'') + msg
    print(msg, end=end)

Level = namedtuple('Level', ['info', 'warning', 'error'])(
    info = u'INFO ',
    warning = u'WARNING ',
    error = u'ERROR '
)

PRINT = 'print'
HTML = 'html'
JSON = 'json'
CSV = 'csv'


class HtmlTemplate:
    '''HTML template.'''
    html = u'''<html>
    <head>
    <meta charset="UTF-8">
    <title>Search Results</title>
    <style>
    body {{ background-color:#f5f5f5; font-family:Italic, Charcoal, sans-serif; }} 
    a:link {{ color: #262626; }} 
    a:visited {{ color: #808080; }} 
    th {{ font-size:17px; text-align:left; padding:3px; font-style: italic; }} 
    td {{ font-size:14px; text-align:left; padding:1px; }} 
    </style>
    </head>
    <body>
    <table>
    <tr><th>Query: '{query}'</th></tr>
    <tr><td> </td></tr>
    </table>
    {table}
    </body>
    </html>
    '''
    table = u'''<table>
    <tr><th>{engine} search results </th></tr>
    </table>
    <table>
    {rows}
    </table>
    <br>
    '''
    row = u'''<tr>
    <td>{number})</td>
    <td><a href="{href}" target="_blank">{link}</a></td>
    {data}
    </tr>
    '''
    data = u'''<tr><td></td><td>{}</td></tr>'''

